'use strict';

import React from "react";
import PropTypes from "prop-types";
import {Glyphicon, Table} from "react-bootstrap";

import Constants from "../../constants/Constants";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import ReportRow from "./ReportRow";
import SortingWrapper from "../misc/SortingWrapper";

class ReportsTable extends React.Component {
    constructor(props) {
        super(props);
        this.i18n = props.i18n;
    }

    render() {
        if (this.props.reports.length === 0) {
            return null;
        }
        return <div>
            <Table striped bordered condensed hover>
                {this._renderHeader()}
                <tbody>
                {this._renderReports()}
                </tbody>
            </Table>
        </div>;
    }

    _renderHeader() {
        return <thead>
        <tr>
            <th className='col-xs-3 content-center table-sorter-wrapper'>
                {this.i18n('headline')}
                {this._renderSortIcon('occurrence.name', 'alphabet')}
            </th>
            <th className='col-xs-2 content-center table-sorter-wrapper'
                title={this.i18n('reports.table-date.tooltip')}>
                {this.i18n('reports.table-date')}
                {this._renderSortIcon('occurrence.startTime', 'order')}
            </th>
            <th className='col-xs-2 content-center'>{this.i18n('reports.table-severity')}</th>
            <th className='col-xs-3 content-center'>{this.i18n('reports.table-classification')}</th>
            <th className='col-xs-2 content-center'>{this.i18n('table-actions')}</th>
        </tr>
        </thead>;
    }

    _renderSortIcon(column, type) {
        const glyph = this.props.sortState[column] ? this.props.sortState[column] : Constants.SORTING.NO;
        let glyphIcon = glyph.glyph;
        if (glyph !== Constants.SORTING.NO) {
            glyphIcon = glyphIcon.replace('$type$', type);
        }
        return <Glyphicon className={'table-sorter-icon column-' + column} glyph={glyphIcon}
                          title={this.i18n(glyph.title)} onClick={() => this.props.onSort(column)}/>;
    }

    _renderReports() {
        const result = [],
            reports = this.props.reports.slice().sort(this.props.sorter);
        for (let i = 0, len = reports.length; i < len; i++) {
            let report = reports[i];
            result.push(<ReportRow report={report} key={report.id} actions={this.props.actions}
                                   types={this.props.types}/>);
        }
        return result;
    }
}

ReportsTable.propTypes = {
    reports: PropTypes.array.isRequired,
    types: PropTypes.array,
    actions: PropTypes.object.isRequired,
    onSort: PropTypes.func.isRequired,
    sorter: PropTypes.func.isRequired,  // This is the function which actually sorts the rows
    sortState: PropTypes.object
};

export default injectIntl(I18nWrapper(SortingWrapper(ReportsTable, {sortAttributes: ['occurrence.name', 'occurrence.startTime']})));
