package cz.cvut.kbss.reporting.environment;

import cz.cvut.kbss.reporting.model.Occurrence;
import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.model.Person;
import cz.cvut.kbss.reporting.model.Severity;
import cz.cvut.kbss.reporting.model.Tag;
import java.net.URI;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

public class Generator {

    public static final Random RANDOM = new Random();
    private static final String TYPE_BASE = "http://onto.fel.cvut.cz/ontologies/aso/event-type";

    private Generator() {
        throw new AssertionError();
    }

    public static Person generatePerson() {
        final Person p = new Person();
        p.setFirstName("Catherine");
        p.setLastName("Halsey");
        p.setUsername("halsey" + RANDOM.nextInt(1000) + "@unsc.org");
        p.setPassword("1Cr3@t3dSp@rt@ns");
        return p;
    }

    public static int randomInt() {
        return RANDOM.nextInt();
    }

    public static int randomInt(int max) {
        return RANDOM.nextInt(max);
    }

    public static int randomInt(int min, int max) {
        assert min < max;
        int diff = randomInt(max - min);
        return min + diff;
    }

    public static boolean randomBoolean() {
        return RANDOM.nextBoolean();
    }

    public static URI generateTypeUri() {
        return URI.create(TYPE_BASE + randomInt());
    }

    public static Occurrence generateOccurrence() {
        final Occurrence occurrence = new Occurrence();
        occurrence.setName("Occurrence" + randomInt());
        occurrence.setStartTime(LocalDateTime
            .ofEpochSecond(System.currentTimeMillis() / 1000 - randomInt(1000, 100000), 0, ZoneOffset.UTC));
        occurrence.setEndTime(LocalDateTime.now());
        occurrence.setSeverity(Severity.values()[randomInt(Severity.values().length)]);
        occurrence.setType(generateTypeUri());
        return occurrence;
    }

    public static OccurrenceReport generateReport() {
        final OccurrenceReport report = new OccurrenceReport();
        report.setOccurrence(generateOccurrence());
        report.setNarrative("This occurrence report has a random narrative with suffix " + randomInt());
        return report;
    }

    public static Tag generateTag(String name) {
        final Tag tag = new Tag();
        tag.setName(name);
        tag.setCreated(LocalDateTime.now());
        return tag;
    }

    public static <T> List<T> getRandomSubset(List<T> list, int minimumSize, int maximumSize) {
        assert minimumSize <= maximumSize;
        List<T> shuffledList = new LinkedList<>(list);
        Collections.shuffle(shuffledList);
        return shuffledList.stream()
            .limit(Generator.randomInt(minimumSize, maximumSize + 1))
            .collect(Collectors.toList());
    }

    public static <T> List<T> getRandomSubset(List<T> list) {
        return getRandomSubset(list, 0, list.size() + 1);
    }
}
